"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EthCallQueryResponse = exports.parseBlockId = exports.EthCallQueryRequest = void 0;
const buffer_1 = require("buffer");
const BinaryWriter_1 = require("./BinaryWriter");
const request_1 = require("./request");
const utils_1 = require("./utils");
const BinaryReader_1 = require("./BinaryReader");
class EthCallQueryRequest {
    constructor(blockTag, callData) {
        this.callData = callData;
        this.blockTag = parseBlockId(blockTag);
    }
    type() {
        return request_1.ChainQueryType.EthCall;
    }
    serialize() {
        const writer = new BinaryWriter_1.BinaryWriter()
            .writeUint32(this.blockTag.length)
            .writeUint8Array(buffer_1.Buffer.from(this.blockTag))
            .writeUint8(this.callData.length);
        this.callData.forEach(({ to, data }) => {
            const dataArray = (0, utils_1.hexToUint8Array)(data);
            writer
                .writeUint8Array((0, utils_1.hexToUint8Array)(to))
                .writeUint32(dataArray.length)
                .writeUint8Array(dataArray);
        });
        return writer.data();
    }
    static from(bytes) {
        const reader = new BinaryReader_1.BinaryReader((0, utils_1.coalesceUint8Array)(bytes));
        return this.fromReader(reader);
    }
    static fromReader(reader) {
        const blockTagLength = reader.readUint32();
        const blockTag = reader.readString(blockTagLength);
        const callDataLength = reader.readUint8();
        const callData = [];
        for (let idx = 0; idx < callDataLength; idx++) {
            const to = reader.readHex(20);
            const dataLength = reader.readUint32();
            const data = reader.readHex(dataLength);
            callData.push({ to, data });
        }
        return new EthCallQueryRequest(blockTag, callData);
    }
}
exports.EthCallQueryRequest = EthCallQueryRequest;
function parseBlockId(blockId) {
    if (!blockId || blockId === "") {
        throw new Error(`block tag is required`);
    }
    if (typeof blockId === "number") {
        if (blockId < 0) {
            throw new Error(`block tag must be positive`);
        }
        blockId = `0x${blockId.toString(16)}`;
    }
    else if ((0, utils_1.isValidHexString)(blockId)) {
        if (!blockId.startsWith("0x")) {
            blockId = `0x${blockId}`;
        }
    }
    else {
        throw new Error(`Invalid block tag: ${blockId}`);
    }
    return blockId;
}
exports.parseBlockId = parseBlockId;
class EthCallQueryResponse {
    constructor(blockNumber, blockHash, blockTime, results = []) {
        this.blockNumber = blockNumber;
        this.blockHash = blockHash;
        this.blockTime = blockTime;
        this.results = results;
    }
    type() {
        return request_1.ChainQueryType.EthCall;
    }
    serialize() {
        const writer = new BinaryWriter_1.BinaryWriter()
            .writeUint64(this.blockNumber)
            .writeUint8Array((0, utils_1.hexToUint8Array)(this.blockHash))
            .writeUint64(this.blockTime)
            .writeUint8(this.results.length);
        for (const result of this.results) {
            const arr = (0, utils_1.hexToUint8Array)(result);
            writer.writeUint32(arr.length).writeUint8Array(arr);
        }
        return writer.data();
    }
    static from(bytes) {
        const reader = new BinaryReader_1.BinaryReader((0, utils_1.coalesceUint8Array)(bytes));
        return this.fromReader(reader);
    }
    static fromReader(reader) {
        const blockNumber = reader.readUint64();
        const blockHash = reader.readHex(32);
        const blockTime = reader.readUint64();
        const resultsLength = reader.readUint8();
        const results = [];
        for (let idx = 0; idx < resultsLength; idx++) {
            const resultLength = reader.readUint32();
            const result = reader.readHex(resultLength);
            results.push(result);
        }
        return new EthCallQueryResponse(blockNumber, blockHash, blockTime, results);
    }
}
exports.EthCallQueryResponse = EthCallQueryResponse;
