"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChainQueryType = exports.PerChainQueryRequest = exports.QueryRequest = exports.getPrefix = exports.DEVNET_QUERY_REQUEST_PREFIX = exports.TESTNET_QUERY_REQUEST_PREFIX = exports.MAINNET_QUERY_REQUEST_PREFIX = void 0;
const keccak256_1 = require("@ethersproject/keccak256");
const buffer_1 = require("buffer");
const BinaryWriter_1 = require("./BinaryWriter");
const utils_1 = require("./utils");
const BinaryReader_1 = require("./BinaryReader");
const ethCall_1 = require("./ethCall");
const ethCallByTimestamp_1 = require("./ethCallByTimestamp");
const ethCallWithFinality_1 = require("./ethCallWithFinality");
const solanaAccount_1 = require("./solanaAccount");
exports.MAINNET_QUERY_REQUEST_PREFIX = "mainnet_query_request_000000000000|";
exports.TESTNET_QUERY_REQUEST_PREFIX = "testnet_query_request_000000000000|";
exports.DEVNET_QUERY_REQUEST_PREFIX = "devnet_query_request_0000000000000|";
function getPrefix(network) {
    return network === "MAINNET"
        ? exports.MAINNET_QUERY_REQUEST_PREFIX
        : network === "TESTNET"
            ? exports.TESTNET_QUERY_REQUEST_PREFIX
            : exports.DEVNET_QUERY_REQUEST_PREFIX;
}
exports.getPrefix = getPrefix;
const REQUEST_VERSION = 1;
class QueryRequest {
    constructor(nonce, requests = [], version = REQUEST_VERSION) {
        this.nonce = nonce;
        this.requests = requests;
        this.version = version;
    }
    serialize() {
        const writer = new BinaryWriter_1.BinaryWriter()
            .writeUint8(this.version)
            .writeUint32(this.nonce)
            .writeUint8(this.requests.length);
        this.requests.forEach((request) => writer.writeUint8Array(request.serialize()));
        return writer.data();
    }
    static digest(network, bytes) {
        const prefix = getPrefix(network);
        const data = buffer_1.Buffer.concat([buffer_1.Buffer.from(prefix), bytes]);
        return (0, utils_1.hexToUint8Array)((0, keccak256_1.keccak256)(data).slice(2));
    }
    static from(bytes) {
        const reader = new BinaryReader_1.BinaryReader((0, utils_1.coalesceUint8Array)(bytes));
        return this.fromReader(reader);
    }
    static fromReader(reader) {
        const version = reader.readUint8();
        if (version != REQUEST_VERSION) {
            throw new Error(`Unsupported message version: ${version}`);
        }
        const nonce = reader.readUint32();
        const queryRequest = new QueryRequest(nonce);
        const numPerChainQueries = reader.readUint8();
        for (let idx = 0; idx < numPerChainQueries; idx++) {
            queryRequest.requests.push(PerChainQueryRequest.fromReader(reader));
        }
        return queryRequest;
    }
}
exports.QueryRequest = QueryRequest;
class PerChainQueryRequest {
    constructor(chainId, query) {
        this.chainId = chainId;
        this.query = query;
    }
    serialize() {
        const writer = new BinaryWriter_1.BinaryWriter()
            .writeUint16(this.chainId)
            .writeUint8(this.query.type());
        const queryData = this.query.serialize();
        return writer
            .writeUint32(queryData.length)
            .writeUint8Array(queryData)
            .data();
    }
    static from(bytes) {
        const reader = new BinaryReader_1.BinaryReader((0, utils_1.coalesceUint8Array)(bytes));
        return this.fromReader(reader);
    }
    static fromReader(reader) {
        const chainId = reader.readUint16();
        const queryType = reader.readUint8();
        reader.readUint32(); // skip the query length
        let query;
        if (queryType === ChainQueryType.EthCall) {
            query = ethCall_1.EthCallQueryRequest.fromReader(reader);
        }
        else if (queryType === ChainQueryType.EthCallByTimeStamp) {
            query = ethCallByTimestamp_1.EthCallByTimestampQueryRequest.fromReader(reader);
        }
        else if (queryType === ChainQueryType.EthCallWithFinality) {
            query = ethCallWithFinality_1.EthCallWithFinalityQueryRequest.fromReader(reader);
        }
        else if (queryType === ChainQueryType.SolanaAccount) {
            query = solanaAccount_1.SolanaAccountQueryRequest.fromReader(reader);
        }
        else {
            throw new Error(`Unsupported query type: ${queryType}`);
        }
        return new PerChainQueryRequest(chainId, query);
    }
}
exports.PerChainQueryRequest = PerChainQueryRequest;
var ChainQueryType;
(function (ChainQueryType) {
    ChainQueryType[ChainQueryType["EthCall"] = 1] = "EthCall";
    ChainQueryType[ChainQueryType["EthCallByTimeStamp"] = 2] = "EthCallByTimeStamp";
    ChainQueryType[ChainQueryType["EthCallWithFinality"] = 3] = "EthCallWithFinality";
    ChainQueryType[ChainQueryType["SolanaAccount"] = 4] = "SolanaAccount";
})(ChainQueryType || (exports.ChainQueryType = ChainQueryType = {}));
