"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PerChainQueryResponse = exports.QueryResponse = exports.QUERY_RESPONSE_PREFIX = void 0;
const keccak256_1 = require("@ethersproject/keccak256");
const buffer_1 = require("buffer");
const utils_1 = require("./utils");
const BinaryReader_1 = require("./BinaryReader");
const request_1 = require("./request");
const BinaryWriter_1 = require("./BinaryWriter");
const ethCall_1 = require("./ethCall");
const ethCallByTimestamp_1 = require("./ethCallByTimestamp");
const ethCallWithFinality_1 = require("./ethCallWithFinality");
const solanaAccount_1 = require("./solanaAccount");
exports.QUERY_RESPONSE_PREFIX = "query_response_0000000000000000000|";
const RESPONSE_VERSION = 1;
class QueryResponse {
    constructor(requestChainId = 0, requestId, request, responses = [], version = RESPONSE_VERSION) {
        this.requestChainId = requestChainId;
        this.requestId = requestId;
        this.request = request;
        this.responses = responses;
        this.version = version;
    }
    serialize() {
        const serializedRequest = this.request.serialize();
        const writer = new BinaryWriter_1.BinaryWriter()
            .writeUint8(this.version)
            .writeUint16(this.requestChainId)
            .writeUint8Array((0, utils_1.hexToUint8Array)(this.requestId)) // TODO: this only works for hex encoded signatures
            .writeUint32(serializedRequest.length)
            .writeUint8Array(serializedRequest)
            .writeUint8(this.responses.length);
        for (const response of this.responses) {
            writer.writeUint8Array(response.serialize());
        }
        return writer.data();
    }
    static digest(bytes) {
        return (0, utils_1.hexToUint8Array)((0, keccak256_1.keccak256)(buffer_1.Buffer.concat([
            buffer_1.Buffer.from(exports.QUERY_RESPONSE_PREFIX),
            (0, utils_1.hexToUint8Array)((0, keccak256_1.keccak256)(bytes)),
        ])));
    }
    static from(bytes) {
        const reader = new BinaryReader_1.BinaryReader((0, utils_1.coalesceUint8Array)(bytes));
        return this.fromReader(reader);
    }
    static fromReader(reader) {
        const version = reader.readUint8();
        if (version != RESPONSE_VERSION) {
            throw new Error(`Unsupported message version: ${version}`);
        }
        const requestChainId = reader.readUint16();
        if (requestChainId !== 0) {
            // TODO: support reading off-chain and on-chain requests
            throw new Error(`Unsupported request chain: ${requestChainId}`);
        }
        const requestId = reader.readHex(65); // signature
        reader.readUint32(); // skip the query length
        const queryRequest = request_1.QueryRequest.fromReader(reader);
        const queryResponse = new QueryResponse(requestChainId, requestId, queryRequest);
        const numPerChainResponses = reader.readUint8();
        for (let idx = 0; idx < numPerChainResponses; idx++) {
            queryResponse.responses.push(PerChainQueryResponse.fromReader(reader));
        }
        return queryResponse;
    }
}
exports.QueryResponse = QueryResponse;
class PerChainQueryResponse {
    constructor(chainId, response) {
        this.chainId = chainId;
        this.response = response;
    }
    serialize() {
        const writer = new BinaryWriter_1.BinaryWriter()
            .writeUint16(this.chainId)
            .writeUint8(this.response.type());
        const queryResponse = this.response.serialize();
        return writer
            .writeUint32(queryResponse.length)
            .writeUint8Array(queryResponse)
            .data();
    }
    static from(bytes) {
        const reader = new BinaryReader_1.BinaryReader((0, utils_1.coalesceUint8Array)(bytes));
        return this.fromReader(reader);
    }
    static fromReader(reader) {
        const chainId = reader.readUint16();
        const queryType = reader.readUint8();
        reader.readUint32(); // skip the query length
        let response;
        if (queryType === request_1.ChainQueryType.EthCall) {
            response = ethCall_1.EthCallQueryResponse.fromReader(reader);
        }
        else if (queryType === request_1.ChainQueryType.EthCallByTimeStamp) {
            response = ethCallByTimestamp_1.EthCallByTimestampQueryResponse.fromReader(reader);
        }
        else if (queryType === request_1.ChainQueryType.EthCallWithFinality) {
            response = ethCallWithFinality_1.EthCallWithFinalityQueryResponse.fromReader(reader);
        }
        else if (queryType === request_1.ChainQueryType.SolanaAccount) {
            response = solanaAccount_1.SolanaAccountQueryResponse.fromReader(reader);
        }
        else {
            throw new Error(`Unsupported response type: ${queryType}`);
        }
        return new PerChainQueryResponse(chainId, response);
    }
}
exports.PerChainQueryResponse = PerChainQueryResponse;
