"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SolanaAccountQueryResponse = exports.SolanaAccountQueryRequest = void 0;
const buffer_1 = require("buffer");
const bs58_1 = __importDefault(require("bs58"));
const BinaryWriter_1 = require("./BinaryWriter");
const request_1 = require("./request");
const utils_1 = require("./utils");
const BinaryReader_1 = require("./BinaryReader");
class SolanaAccountQueryRequest {
    constructor(commitment, accounts, minContextSlot, dataSliceOffset, dataSliceLength) {
        this.commitment = commitment;
        this.minContextSlot = bigIntWithDef(minContextSlot);
        this.dataSliceOffset = bigIntWithDef(dataSliceOffset);
        this.dataSliceLength = bigIntWithDef(dataSliceLength);
        this.accounts = [];
        accounts.forEach((account) => {
            if (account.startsWith("0x")) {
                // Should be 32 bytes.
                if (account.length != 66) {
                    throw new Error(`Invalid account, must be 32 bytes: ${account}`);
                }
                this.accounts.push(Uint8Array.from(buffer_1.Buffer.from(account.substring(2), "hex")));
            }
            else {
                // Should be base58.
                this.accounts.push(Uint8Array.from(bs58_1.default.decode(account)));
            }
        });
    }
    type() {
        return request_1.ChainQueryType.SolanaAccount;
    }
    serialize() {
        const writer = new BinaryWriter_1.BinaryWriter()
            .writeUint32(this.commitment.length)
            .writeUint8Array(buffer_1.Buffer.from(this.commitment))
            .writeUint64(this.minContextSlot)
            .writeUint64(this.dataSliceOffset)
            .writeUint64(this.dataSliceLength)
            .writeUint8(this.accounts.length);
        this.accounts.forEach((account) => {
            writer.writeUint8Array(account);
        });
        return writer.data();
    }
    static from(bytes) {
        const reader = new BinaryReader_1.BinaryReader((0, utils_1.coalesceUint8Array)(bytes));
        return this.fromReader(reader);
    }
    static fromReader(reader) {
        const commitmentLength = reader.readUint32();
        const commitment = reader.readString(commitmentLength);
        if (commitment !== "finalized") {
            throw new Error(`Invalid commitment: ${commitment}`);
        }
        const minContextSlot = reader.readUint64();
        const dataSliceOffset = reader.readUint64();
        const dataSliceLength = reader.readUint64();
        const numAccounts = reader.readUint8();
        const accounts = [];
        for (let idx = 0; idx < numAccounts; idx++) {
            const account = reader.readUint8Array(32);
            // Add the "0x" prefix so the constructor knows it's hex rather than base58.
            accounts.push("0x" + buffer_1.Buffer.from(account).toString("hex"));
        }
        return new SolanaAccountQueryRequest(commitment, accounts, minContextSlot, dataSliceOffset, dataSliceLength);
    }
}
exports.SolanaAccountQueryRequest = SolanaAccountQueryRequest;
class SolanaAccountQueryResponse {
    constructor(slotNumber, blockTime, blockHash, results) {
        if (blockHash.length != 32) {
            throw new Error(`Invalid block hash, should be 32 bytes long: ${blockHash}`);
        }
        for (const result of results) {
            if (result.owner.length != 32) {
                throw new Error(`Invalid owner, should be 32 bytes long: ${result.owner}`);
            }
        }
        this.slotNumber = slotNumber;
        this.blockTime = blockTime;
        this.blockHash = blockHash;
        this.results = results;
    }
    type() {
        return request_1.ChainQueryType.SolanaAccount;
    }
    serialize() {
        const writer = new BinaryWriter_1.BinaryWriter()
            .writeUint64(this.slotNumber)
            .writeUint64(this.blockTime)
            .writeUint8Array(this.blockHash)
            .writeUint8(this.results.length);
        for (const result of this.results) {
            writer
                .writeUint64(result.lamports)
                .writeUint64(result.rentEpoch)
                .writeUint8(result.executable ? 1 : 0)
                .writeUint8Array(result.owner)
                .writeUint32(result.data.length)
                .writeUint8Array(result.data);
        }
        return writer.data();
    }
    static from(bytes) {
        const reader = new BinaryReader_1.BinaryReader((0, utils_1.coalesceUint8Array)(bytes));
        return this.fromReader(reader);
    }
    static fromReader(reader) {
        const slotNumber = reader.readUint64();
        const blockTime = reader.readUint64();
        const blockHash = reader.readUint8Array(32);
        const resultsLength = reader.readUint8();
        const results = [];
        for (let idx = 0; idx < resultsLength; idx++) {
            const lamports = reader.readUint64();
            const rentEpoch = reader.readUint64();
            const executableU8 = reader.readUint8();
            const executable = executableU8 != 0;
            const owner = reader.readUint8Array(32);
            const dataLength = reader.readUint32();
            const data = reader.readUint8Array(dataLength);
            results.push({ lamports, rentEpoch, executable, owner, data });
        }
        return new SolanaAccountQueryResponse(slotNumber, blockTime, blockHash, results);
    }
}
exports.SolanaAccountQueryResponse = SolanaAccountQueryResponse;
function bigIntWithDef(val) {
    return BigInt(val !== undefined ? val : BigInt(0));
}
