var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import axios from "axios";
import base58 from "bs58";
import { Buffer } from "buffer";
import { ChainQueryType, EthCallByTimestampQueryResponse, EthCallQueryResponse, EthCallWithFinalityQueryResponse, PerChainQueryResponse, QueryResponse, sign, SolanaAccountQueryResponse, } from "../query";
/**
 * Usage:
 *
 * ```js
 * const mock = new QueryProxyMock({
 *   2: "http://localhost:8545",
 * });
 * ```
 *
 * If you are running an Anvil fork like
 *
 * ```bash
 * anvil -f https://ethereum-goerli.publicnode.com
 * ```
 *
 * You can use the following command to switch the guardian address to the devnet / mock guardian
 *
 * Where the `-a` parameter is the core bridge address on that chain
 *
 * https://docs.wormhole.com/wormhole/reference/constants#core-contracts
 *
 * ```bash
 * npx \@wormhole-foundation/wormhole-cli evm hijack -a 0x706abc4E45D419950511e474C7B9Ed348A4a716c -g 0xbeFA429d57cD18b7F8A4d91A2da9AB4AF05d0FBe
 * ```
 */
export class QueryProxyMock {
    constructor(rpcMap, mockPrivateKeys = [
        "cfb12303a19cde580bb4dd771639b0d26bc68353645571a8cff516ab2ee113a0",
    ]) {
        this.rpcMap = rpcMap;
        this.mockPrivateKeys = mockPrivateKeys;
    }
    sign(serializedResponse) {
        const digest = QueryResponse.digest(serializedResponse);
        return this.mockPrivateKeys.map((key, idx) => `${sign(key, digest)}${idx.toString(16).padStart(2, "0")}`);
    }
    /**
     * Usage:
     *
     * ```js
     * const { bytes, signatures } = await mock.mock(new QueryRequest(nonce, [
     *   new PerChainQueryRequest(
     *     wormholeChainId,
     *     new EthCallQueryRequest(blockNumber, [
     *       { to: contract, data: abiEncodedData },
     *     ])
     *   ),
     * ]));
     * ```
     *
     * @param queryRequest an instance of the `QueryRequest` class
     * @returns a promise result matching the query proxy's query response
     */
    mock(queryRequest) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p;
        return __awaiter(this, void 0, void 0, function* () {
            const queryResponse = new QueryResponse(0, // source = off-chain
            Buffer.from(new Array(65)).toString("hex"), // empty signature for mock
            queryRequest);
            for (const perChainRequest of queryRequest.requests) {
                const rpc = this.rpcMap[perChainRequest.chainId];
                if (!rpc) {
                    throw new Error(`Unregistered chain id for mock: ${perChainRequest.chainId}`);
                }
                const type = perChainRequest.query.type();
                if (type === ChainQueryType.EthCall) {
                    const query = perChainRequest.query;
                    const response = yield axios.post(rpc, [
                        ...query.callData.map((args, idx) => ({
                            jsonrpc: "2.0",
                            id: idx,
                            method: "eth_call",
                            params: [
                                args,
                                //TODO: support block hash
                                query.blockTag,
                            ],
                        })),
                        {
                            jsonrpc: "2.0",
                            id: query.callData.length,
                            //TODO: support block hash
                            method: "eth_getBlockByNumber",
                            params: [query.blockTag, false],
                        },
                    ]);
                    const callResults = (_a = response === null || response === void 0 ? void 0 : response.data) === null || _a === void 0 ? void 0 : _a.slice(0, query.callData.length);
                    const blockResult = (_c = (_b = response === null || response === void 0 ? void 0 : response.data) === null || _b === void 0 ? void 0 : _b[query.callData.length]) === null || _c === void 0 ? void 0 : _c.result;
                    if (!blockResult ||
                        !blockResult.number ||
                        !blockResult.timestamp ||
                        !blockResult.hash) {
                        throw new Error(`Invalid block result for chain ${perChainRequest.chainId} block ${query.blockTag}`);
                    }
                    queryResponse.responses.push(new PerChainQueryResponse(perChainRequest.chainId, new EthCallQueryResponse(BigInt(parseInt(blockResult.number.substring(2), 16)), // block number
                    blockResult.hash, // hash
                    BigInt(parseInt(blockResult.timestamp.substring(2), 16)) *
                        BigInt("1000000"), // time in seconds -> microseconds
                    callResults.map((callResult) => callResult.result))));
                }
                else if (type === ChainQueryType.EthCallWithFinality) {
                    const query = perChainRequest.query;
                    const response = yield axios.post(rpc, [
                        ...query.callData.map((args, idx) => ({
                            jsonrpc: "2.0",
                            id: idx,
                            method: "eth_call",
                            params: [
                                args,
                                //TODO: support block hash
                                query.blockId,
                            ],
                        })),
                        {
                            jsonrpc: "2.0",
                            id: query.callData.length,
                            //TODO: support block hash
                            method: "eth_getBlockByNumber",
                            params: [query.blockId, false],
                        },
                        {
                            jsonrpc: "2.0",
                            id: query.callData.length,
                            method: "eth_getBlockByNumber",
                            params: [query.finality, false],
                        },
                    ]);
                    const callResults = (_d = response === null || response === void 0 ? void 0 : response.data) === null || _d === void 0 ? void 0 : _d.slice(0, query.callData.length);
                    const blockResult = (_f = (_e = response === null || response === void 0 ? void 0 : response.data) === null || _e === void 0 ? void 0 : _e[query.callData.length]) === null || _f === void 0 ? void 0 : _f.result;
                    const finalityBlockResult = (_h = (_g = response === null || response === void 0 ? void 0 : response.data) === null || _g === void 0 ? void 0 : _g[query.callData.length + 1]) === null || _h === void 0 ? void 0 : _h.result;
                    if (!blockResult ||
                        !blockResult.number ||
                        !blockResult.timestamp ||
                        !blockResult.hash) {
                        throw new Error(`Invalid block result for chain ${perChainRequest.chainId} block ${query.blockId}`);
                    }
                    if (!finalityBlockResult ||
                        !finalityBlockResult.number ||
                        !finalityBlockResult.timestamp ||
                        !finalityBlockResult.hash) {
                        throw new Error(`Invalid tagged block result for chain ${perChainRequest.chainId} tag ${query.finality}`);
                    }
                    const blockNumber = BigInt(parseInt(blockResult.number.substring(2), 16));
                    const latestBlockNumberMatchingFinality = BigInt(parseInt(finalityBlockResult.number.substring(2), 16));
                    if (blockNumber > latestBlockNumberMatchingFinality) {
                        throw new Error(`Requested block for eth_call_with_finality has not yet reached the requested finality. Block: ${blockNumber}, ${query.finality}: ${latestBlockNumberMatchingFinality}`);
                    }
                    queryResponse.responses.push(new PerChainQueryResponse(perChainRequest.chainId, new EthCallWithFinalityQueryResponse(BigInt(parseInt(blockResult.number.substring(2), 16)), // block number
                    blockResult.hash, // hash
                    BigInt(parseInt(blockResult.timestamp.substring(2), 16)) *
                        BigInt("1000000"), // time in seconds -> microseconds
                    callResults.map((callResult) => callResult.result))));
                }
                else if (type === ChainQueryType.EthCallByTimeStamp) {
                    const query = perChainRequest.query;
                    // Verify that the two block hints are consistent, either both set, or both unset.
                    if ((query.targetBlockHint === "") !==
                        (query.followingBlockHint === "")) {
                        throw new Error(`Invalid block id hints in eth_call_by_timestamp query request, both should be either set or unset`);
                    }
                    let targetBlock = query.targetBlockHint;
                    let followingBlock = query.followingBlockHint;
                    if (targetBlock === "") {
                        let nextQueryBlock = "latest";
                        let tries = 0;
                        let targetTimestamp = BigInt(0);
                        let followingTimestamp = BigInt(0);
                        while (query.targetTimestamp < targetTimestamp ||
                            query.targetTimestamp >= followingTimestamp) {
                            if (tries > 128) {
                                throw new Error(`Timestamp was not within the last 128 blocks.`);
                            }
                            // TODO: batching
                            const blockResult = (_j = (yield axios.post(rpc, {
                                jsonrpc: "2.0",
                                id: 1,
                                method: "eth_getBlockByNumber",
                                params: [nextQueryBlock, false],
                            })).data) === null || _j === void 0 ? void 0 : _j.result;
                            if (!blockResult) {
                                throw new Error(`Invalid block result while searching for timestamp of ${nextQueryBlock}`);
                            }
                            followingBlock = targetBlock;
                            followingTimestamp = targetTimestamp;
                            targetBlock = blockResult.number;
                            targetTimestamp =
                                BigInt(parseInt(blockResult.timestamp.substring(2), 16)) *
                                    BigInt("1000000"); // time in seconds -> microseconds
                            nextQueryBlock = `0x${(BigInt(blockResult.number) - BigInt(1)).toString(16)}`;
                            tries++;
                        }
                    }
                    const response = yield axios.post(rpc, [
                        ...query.callData.map((args, idx) => ({
                            jsonrpc: "2.0",
                            id: idx,
                            method: "eth_call",
                            params: [
                                args,
                                //TODO: support block hash
                                targetBlock,
                            ],
                        })),
                        {
                            jsonrpc: "2.0",
                            id: query.callData.length,
                            method: "eth_getBlockByNumber",
                            params: [targetBlock, false],
                        },
                        {
                            jsonrpc: "2.0",
                            id: query.callData.length,
                            method: "eth_getBlockByNumber",
                            params: [followingBlock, false],
                        },
                    ]);
                    const callResults = (_k = response === null || response === void 0 ? void 0 : response.data) === null || _k === void 0 ? void 0 : _k.slice(0, query.callData.length);
                    const targetBlockResult = (_m = (_l = response === null || response === void 0 ? void 0 : response.data) === null || _l === void 0 ? void 0 : _l[query.callData.length]) === null || _m === void 0 ? void 0 : _m.result;
                    const followingBlockResult = (_p = (_o = response === null || response === void 0 ? void 0 : response.data) === null || _o === void 0 ? void 0 : _o[query.callData.length + 1]) === null || _p === void 0 ? void 0 : _p.result;
                    if (!targetBlockResult ||
                        !targetBlockResult.number ||
                        !targetBlockResult.timestamp ||
                        !targetBlockResult.hash) {
                        throw new Error(`Invalid target block result for chain ${perChainRequest.chainId} block ${query.targetBlockHint}`);
                    }
                    if (!followingBlockResult ||
                        !followingBlockResult.number ||
                        !followingBlockResult.timestamp ||
                        !followingBlockResult.hash) {
                        throw new Error(`Invalid following block result for chain ${perChainRequest.chainId} tag ${query.followingBlockHint}`);
                    }
                    /*
                      target_block.timestamp <= target_time < following_block.timestamp
                      and
                      following_block_num - 1 == target_block_num
                    */
                    const targetBlockNumber = BigInt(parseInt(targetBlockResult.number.substring(2), 16));
                    const followingBlockNumber = BigInt(parseInt(followingBlockResult.number.substring(2), 16));
                    if (targetBlockNumber + BigInt(1) !== followingBlockNumber) {
                        throw new Error(`eth_call_by_timestamp query blocks are not adjacent`);
                    }
                    const targetTimestamp = BigInt(parseInt(targetBlockResult.timestamp.substring(2), 16)) *
                        BigInt("1000000"); // time in seconds -> microseconds
                    const followingTimestamp = BigInt(parseInt(followingBlockResult.timestamp.substring(2), 16)) *
                        BigInt("1000000"); // time in seconds -> microseconds
                    if (query.targetTimestamp < targetTimestamp ||
                        query.targetTimestamp >= followingTimestamp) {
                        throw new Error(`eth_call_by_timestamp desired timestamp falls outside of block range`);
                    }
                    queryResponse.responses.push(new PerChainQueryResponse(perChainRequest.chainId, new EthCallByTimestampQueryResponse(BigInt(parseInt(targetBlockResult.number.substring(2), 16)), // block number
                    targetBlockResult.hash, // hash
                    targetTimestamp, BigInt(parseInt(followingBlockResult.number.substring(2), 16)), // block number
                    followingBlockResult.hash, // hash
                    followingTimestamp, callResults.map((callResult) => callResult.result))));
                }
                else if (type === ChainQueryType.SolanaAccount) {
                    const query = perChainRequest.query;
                    // Validate the request.
                    if (query.commitment !== "finalized") {
                        throw new Error(`Invalid commitment in sol_account query request, must be "finalized"`);
                    }
                    if (query.dataSliceLength === BigInt(0) &&
                        query.dataSliceOffset !== BigInt(0)) {
                        throw new Error(`data slice offset may not be set if data slice length is zero`);
                    }
                    if (query.accounts.length <= 0) {
                        throw new Error(`does not contain any account entries`);
                    }
                    if (query.accounts.length > 255) {
                        throw new Error(`too many account entries`);
                    }
                    let accounts = [];
                    query.accounts.forEach((acct) => {
                        if (acct.length != 32) {
                            throw new Error(`invalid account length`);
                        }
                        accounts.push(base58.encode(acct));
                    });
                    let opts = {
                        commitment: query.commitment,
                    };
                    if (query.minContextSlot != BigInt(0)) {
                        opts.minContextSlot = Number(query.minContextSlot);
                    }
                    if (query.dataSliceLength !== BigInt(0)) {
                        opts.dataSlice = {
                            offset: Number(query.dataSliceOffset),
                            length: Number(query.dataSliceLength),
                        };
                    }
                    const response = yield axios.post(rpc, {
                        jsonrpc: "2.0",
                        id: 1,
                        method: "getMultipleAccounts",
                        params: [accounts, opts],
                    });
                    if (!response.data.result) {
                        throw new Error("Invalid result for getMultipleAccounts");
                    }
                    const slotNumber = response.data.result.context.slot;
                    let results = [];
                    response.data.result.value.forEach((val) => {
                        results.push({
                            lamports: BigInt(val.lamports),
                            rentEpoch: BigInt(val.rentEpoch),
                            executable: Boolean(val.executable),
                            owner: Uint8Array.from(base58.decode(val.owner.toString())),
                            data: Uint8Array.from(Buffer.from(val.data[0].toString(), "base64")),
                        });
                    });
                    const response2 = yield axios.post(rpc, {
                        jsonrpc: "2.0",
                        id: 1,
                        method: "getBlock",
                        params: [
                            slotNumber,
                            { commitment: query.commitment, transactionDetails: "none" },
                        ],
                    });
                    const blockTime = response2.data.result.blockTime;
                    const blockHash = base58.decode(response2.data.result.blockhash);
                    queryResponse.responses.push(new PerChainQueryResponse(perChainRequest.chainId, new SolanaAccountQueryResponse(BigInt(slotNumber), BigInt(blockTime) * BigInt(1000000), // time in seconds -> microseconds,
                    blockHash, results)));
                }
                else {
                    throw new Error(`Unsupported query type for mock: ${type}`);
                }
            }
            const serializedResponse = queryResponse.serialize();
            return {
                signatures: this.sign(serializedResponse),
                bytes: Buffer.from(serializedResponse).toString("hex"),
            };
        });
    }
}
