import { Buffer } from "buffer";
import { BinaryWriter } from "./BinaryWriter";
import { ChainQueryType } from "./request";
import { coalesceUint8Array, hexToUint8Array, isValidHexString } from "./utils";
import { BinaryReader } from "./BinaryReader";
export class EthCallQueryRequest {
    constructor(blockTag, callData) {
        this.callData = callData;
        this.blockTag = parseBlockId(blockTag);
    }
    type() {
        return ChainQueryType.EthCall;
    }
    serialize() {
        const writer = new BinaryWriter()
            .writeUint32(this.blockTag.length)
            .writeUint8Array(Buffer.from(this.blockTag))
            .writeUint8(this.callData.length);
        this.callData.forEach(({ to, data }) => {
            const dataArray = hexToUint8Array(data);
            writer
                .writeUint8Array(hexToUint8Array(to))
                .writeUint32(dataArray.length)
                .writeUint8Array(dataArray);
        });
        return writer.data();
    }
    static from(bytes) {
        const reader = new BinaryReader(coalesceUint8Array(bytes));
        return this.fromReader(reader);
    }
    static fromReader(reader) {
        const blockTagLength = reader.readUint32();
        const blockTag = reader.readString(blockTagLength);
        const callDataLength = reader.readUint8();
        const callData = [];
        for (let idx = 0; idx < callDataLength; idx++) {
            const to = reader.readHex(20);
            const dataLength = reader.readUint32();
            const data = reader.readHex(dataLength);
            callData.push({ to, data });
        }
        return new EthCallQueryRequest(blockTag, callData);
    }
}
export function parseBlockId(blockId) {
    if (!blockId || blockId === "") {
        throw new Error(`block tag is required`);
    }
    if (typeof blockId === "number") {
        if (blockId < 0) {
            throw new Error(`block tag must be positive`);
        }
        blockId = `0x${blockId.toString(16)}`;
    }
    else if (isValidHexString(blockId)) {
        if (!blockId.startsWith("0x")) {
            blockId = `0x${blockId}`;
        }
    }
    else {
        throw new Error(`Invalid block tag: ${blockId}`);
    }
    return blockId;
}
export class EthCallQueryResponse {
    constructor(blockNumber, blockHash, blockTime, results = []) {
        this.blockNumber = blockNumber;
        this.blockHash = blockHash;
        this.blockTime = blockTime;
        this.results = results;
    }
    type() {
        return ChainQueryType.EthCall;
    }
    serialize() {
        const writer = new BinaryWriter()
            .writeUint64(this.blockNumber)
            .writeUint8Array(hexToUint8Array(this.blockHash))
            .writeUint64(this.blockTime)
            .writeUint8(this.results.length);
        for (const result of this.results) {
            const arr = hexToUint8Array(result);
            writer.writeUint32(arr.length).writeUint8Array(arr);
        }
        return writer.data();
    }
    static from(bytes) {
        const reader = new BinaryReader(coalesceUint8Array(bytes));
        return this.fromReader(reader);
    }
    static fromReader(reader) {
        const blockNumber = reader.readUint64();
        const blockHash = reader.readHex(32);
        const blockTime = reader.readUint64();
        const resultsLength = reader.readUint8();
        const results = [];
        for (let idx = 0; idx < resultsLength; idx++) {
            const resultLength = reader.readUint32();
            const result = reader.readHex(resultLength);
            results.push(result);
        }
        return new EthCallQueryResponse(blockNumber, blockHash, blockTime, results);
    }
}
